"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const lang_1 = require("../core/lang");
const exception_1 = require("../core/exception");
function isCollection(c) {
    if (!(c instanceof AbstractCollection)) {
        throw new TypeError(`Expect type Collection, but got ${typeof c}`);
    }
}
exports.isCollection = isCollection;
class AbstractCollection {
    add(e, index) {
        throw new exception_1.UnsupportedOperationException();
    }
    addAll(c, index) {
        lang_1.Assert.requireNotNull(c);
        isCollection(c);
        if (index) {
            throw new exception_1.UnsupportedOperationException();
        }
        let it = c.iterator();
        while (it.hasNext()) {
            this.add(it.next());
        }
    }
    clear() {
        let it = this.iterator();
        while (it.hasNext()) {
            it.next();
            it.remove();
        }
    }
    contains(o) {
        let it = this.iterator();
        while (it.hasNext()) {
            if (it.next() === o) {
                return true;
            }
        }
        return false;
    }
    containsAll(c) {
        lang_1.Assert.requireNotNull(c);
        isCollection(c);
        let all = true;
        for (let elem of c) {
            if (!(all &= this.contains(elem))) {
                break;
            }
        }
        return all;
    }
    isEmpty() {
        return this.size() <= 0;
    }
    remove(o) {
        let it = this.iterator();
        while (it.hasNext()) {
            if (it.next() === o) {
                it.remove();
                return true;
            }
        }
        return false;
    }
    removeAll(c) {
        lang_1.Assert.requireNotNull(c);
        isCollection(c);
        let modified = false, it = c.iterator();
        while (it.hasNext()) {
            if (this.contains(it.next())) {
                modified = true;
                it.remove();
            }
        }
        return true;
    }
    toArray() {
        let elements = [];
        let it = this.iterator();
        while (it.hasNext()) {
            elements.push(it.next());
        }
        return elements;
    }
    [Symbol.iterator]() {
        let it = this.iterator();
        return new class {
            next() {
                let done = !it.hasNext();
                return { done, value: done ? undefined : it.next() };
            }
            hasNext() {
                return it.hasNext();
            }
            remove() {
                // ignore
            }
        };
    }
    forEach(callbackFn) {
        if (!(callbackFn instanceof Function)) {
            throw new TypeError(`Expect type function, but got ${typeof callbackFn}`);
        }
        let index = 0, it = this.iterator();
        while (it.hasNext()) {
            if (true === callbackFn(it.next(), index++)) {
                break;
            }
        }
    }
    toString() {
        return JSON.stringify(this.toArray());
    }
}
exports.AbstractCollection = AbstractCollection;
//# sourceMappingURL=collection.js.map